@@ -3,11 +3,8 @@
 #![allow(clippy::module_name_repetitions)]
 
 use std::borrow::Cow;
-use std::ops::ControlFlow;
 use std::{collections::BTreeMap, convert::TryFrom, mem, ops::Index, ops::IndexMut};
 
-#[cfg(feature = "encoding")]
-use encoding_rs::{Decoder, DecoderResult, Encoding};
 use hashlink::LinkedHashMap;
 
 use crate::parser::{Event, MarkedEventReceiver, Parser, Tag};
@@ -68,7 +65,10 @@ fn parse_f64(v: &str) -> Option<f64> {
         ".inf" | ".Inf" | ".INF" | "+.inf" | "+.Inf" | "+.INF" => Some(f64::INFINITY),
         "-.inf" | "-.Inf" | "-.INF" => Some(f64::NEG_INFINITY),
         ".nan" | ".NaN" | ".NAN" => Some(f64::NAN),
-        _ => v.parse::<f64>().ok(),
+        // Test that `v` contains a digit so as not to pass in strings like `inf`,
+        // which rust will parse as a float
+        _ if v.as_bytes().iter().any(u8::is_ascii_digit) => v.parse::<f64>().ok(),
+        _ => None,
     }
 }
 
@@ -107,7 +107,7 @@ pub enum LoadError {
     /// An error within the scanner. This indicates a malformed YAML input.
     Scan(ScanError),
     /// A decoding error (e.g.: Invalid UTF-8).
-    Decode(std::borrow::Cow<'static, str>),
+    Decode(Cow<'static, str>),
 }
 
 impl From<std::io::Error> for LoadError {
@@ -157,13 +157,11 @@ impl YamlLoader {
                 {
                     if handle == "tag:yaml.org,2002:" {
                         match suffix.as_ref() {
-                            "bool" => {
-                                // "true" or "false"
-                                match v.parse::<bool>() {
-                                    Err(_) => Yaml::BadValue,
-                                    Ok(v) => Yaml::Boolean(v),
-                                }
-                            }
+                            "bool" => match v.as_str() {
+                                "true" | "True" | "TRUE" => Yaml::Boolean(true),
+                                "false" | "False" | "FALSE" => Yaml::Boolean(false),
+                                _ => Yaml::BadValue,
+                            },
                             "int" => match v.parse::<i64>() {
                                 Err(_) => Yaml::BadValue,
                                 Ok(v) => Yaml::Integer(v),
@@ -283,195 +281,218 @@ impl YamlLoader {
     }
 }
 
-/// The signature of the function to call when using [`YAMLDecodingTrap::Call`].
-///
-/// The arguments are as follows:
-///  * `malformation_length`: The length of the sequence the decoder failed to decode.
-///  * `bytes_read_after_malformation`: The number of lookahead bytes the decoder consumed after
-///    the malformation.
-///  * `input_at_malformation`: What the input buffer is at the malformation.
-///    This is the buffer starting at the malformation. The first `malformation_length` bytes are
-///    the problematic sequence. The following `bytes_read_after_malformation` are already stored
-///    in the decoder and will not be re-fed.
-///  * `output`: The output string.
-///
-/// The function must modify `output` as it feels is best. For instance, one could recreate the
-/// behavior of [`YAMLDecodingTrap::Ignore`] with an empty function, [`YAMLDecodingTrap::Replace`]
-/// by pushing a `\u{FFFD}` into `output` and [`YAMLDecodingTrap::Strict`] by returning
-/// [`ControlFlow::Break`].
-///
-/// # Returns
-/// The function must return [`ControlFlow::Continue`] if decoding may continue or
-/// [`ControlFlow::Break`] if decoding must be aborted. An optional error string may be supplied.
 #[cfg(feature = "encoding")]
-pub type YAMLDecodingTrapFn = fn(
-    malformation_length: u8,
-    bytes_read_after_malformation: u8,
-    input_at_malformation: &[u8],
-    output: &mut String,
-) -> ControlFlow<Cow<'static, str>>;
-
-/// The behavior [`YamlDecoder`] must have when an decoding error occurs.
-#[cfg(feature = "encoding")]
-#[derive(Copy, Clone, PartialEq, Eq)]
-pub enum YAMLDecodingTrap {
-    /// Ignore the offending bytes, remove them from the output.
-    Ignore,
-    /// Error out.
-    Strict,
-    /// Replace them with the Unicode REPLACEMENT CHARACTER.
-    Replace,
-    /// Call the user-supplied function upon decoding malformation.
-    Call(YAMLDecodingTrapFn),
-}
-
-/// `YamlDecoder` is a `YamlLoader` builder that allows you to supply your own encoding error trap.
-/// For example, to read a YAML file while ignoring Unicode decoding errors you can set the
-/// `encoding_trap` to `encoding::DecoderTrap::Ignore`.
-/// ```rust
-/// use yaml_rust2::yaml::{YamlDecoder, YAMLDecodingTrap};
-///
-/// let string = b"---
-/// a\xa9: 1
-/// b: 2.2
-/// c: [1, 2]
-/// ";
-/// let out = YamlDecoder::read(string as &[u8])
-///     .encoding_trap(YAMLDecodingTrap::Ignore)
-///     .decode()
-///     .unwrap();
-/// ```
-#[cfg(feature = "encoding")]
-pub struct YamlDecoder<T: std::io::Read> {
-    source: T,
-    trap: YAMLDecodingTrap,
-}
+pub use encoding::{YAMLDecodingTrap, YAMLDecodingTrapFn, YamlDecoder};
 
 #[cfg(feature = "encoding")]
-impl<T: std::io::Read> YamlDecoder<T> {
-    /// Create a `YamlDecoder` decoding the given source.
-    pub fn read(source: T) -> YamlDecoder<T> {
-        YamlDecoder {
-            source,
-            trap: YAMLDecodingTrap::Strict,
+mod encoding {
+    use std::{borrow::Cow, ops::ControlFlow};
+
+    use encoding_rs::{Decoder, DecoderResult, Encoding};
+
+    use crate::yaml::{LoadError, Yaml, YamlLoader};
+
+    /// The signature of the function to call when using [`YAMLDecodingTrap::Call`].
+    ///
+    /// The arguments are as follows:
+    ///  * `malformation_length`: The length of the sequence the decoder failed to decode.
+    ///  * `bytes_read_after_malformation`: The number of lookahead bytes the decoder consumed after
+    ///    the malformation.
+    ///  * `input_at_malformation`: What the input buffer is at the malformation.
+    ///    This is the buffer starting at the malformation. The first `malformation_length` bytes are
+    ///    the problematic sequence. The following `bytes_read_after_malformation` are already stored
+    ///    in the decoder and will not be re-fed.
+    ///  * `output`: The output string.
+    ///
+    /// The function must modify `output` as it feels is best. For instance, one could recreate the
+    /// behavior of [`YAMLDecodingTrap::Ignore`] with an empty function, [`YAMLDecodingTrap::Replace`]
+    /// by pushing a `\u{FFFD}` into `output` and [`YAMLDecodingTrap::Strict`] by returning
+    /// [`ControlFlow::Break`].
+    ///
+    /// # Returns
+    /// The function must return [`ControlFlow::Continue`] if decoding may continue or
+    /// [`ControlFlow::Break`] if decoding must be aborted. An optional error string may be supplied.
+    pub type YAMLDecodingTrapFn = fn(
+        malformation_length: u8,
+        bytes_read_after_malformation: u8,
+        input_at_malformation: &[u8],
+        output: &mut String,
+    ) -> ControlFlow<Cow<'static, str>>;
+
+    /// The behavior [`YamlDecoder`] must have when an decoding error occurs.
+    #[derive(Copy, Clone)]
+    pub enum YAMLDecodingTrap {
+        /// Ignore the offending bytes, remove them from the output.
+        Ignore,
+        /// Error out.
+        Strict,
+        /// Replace them with the Unicode REPLACEMENT CHARACTER.
+        Replace,
+        /// Call the user-supplied function upon decoding malformation.
+        Call(YAMLDecodingTrapFn),
+    }
+
+    impl PartialEq for YAMLDecodingTrap {
+        fn eq(&self, other: &YAMLDecodingTrap) -> bool {
+            match (self, other) {
+                (YAMLDecodingTrap::Call(self_fn), YAMLDecodingTrap::Call(other_fn)) => {
+                    *self_fn as usize == *other_fn as usize
+                }
+                (x, y) => x == y,
+            }
         }
     }
 
-    /// Set the behavior of the decoder when the encoding is invalid.
-    pub fn encoding_trap(&mut self, trap: YAMLDecodingTrap) -> &mut Self {
-        self.trap = trap;
-        self
-    }
+    impl Eq for YAMLDecodingTrap {}
 
-    /// Run the decode operation with the source and trap the `YamlDecoder` was built with.
+    /// `YamlDecoder` is a `YamlLoader` builder that allows you to supply your own encoding error trap.
+    /// For example, to read a YAML file while ignoring Unicode decoding errors you can set the
+    /// `encoding_trap` to `encoding::DecoderTrap::Ignore`.
+    /// ```rust
+    /// use yaml_rust2::yaml::{YamlDecoder, YAMLDecodingTrap};
     ///
-    /// # Errors
-    /// Returns `LoadError` when decoding fails.
-    pub fn decode(&mut self) -> Result<Vec<Yaml>, LoadError> {
-        let mut buffer = Vec::new();
-        self.source.read_to_end(&mut buffer)?;
-
-        // Check if the `encoding` library can detect encoding from the BOM, otherwise use
-        // `detect_utf16_endianness`.
-        let (encoding, _) =
-            Encoding::for_bom(&buffer).unwrap_or_else(|| (detect_utf16_endianness(&buffer), 2));
-        let mut decoder = encoding.new_decoder();
-        let mut output = String::new();
-
-        // Decode the input buffer.
-        decode_loop(&buffer, &mut output, &mut decoder, self.trap)?;
-
-        YamlLoader::load_from_str(&output).map_err(LoadError::Scan)
+    /// let string = b"---
+    /// a\xa9: 1
+    /// b: 2.2
+    /// c: [1, 2]
+    /// ";
+    /// let out = YamlDecoder::read(string as &[u8])
+    ///     .encoding_trap(YAMLDecodingTrap::Ignore)
+    ///     .decode()
+    ///     .unwrap();
+    /// ```
+    pub struct YamlDecoder<T: std::io::Read> {
+        source: T,
+        trap: YAMLDecodingTrap,
     }
-}
 
-/// Perform a loop of [`Decoder::decode_to_string`], reallocating `output` if needed.
-#[cfg(feature = "encoding")]
-fn decode_loop(
-    input: &[u8],
-    output: &mut String,
-    decoder: &mut Decoder,
-    trap: YAMLDecodingTrap,
-) -> Result<(), LoadError> {
-    output.reserve(input.len());
-    let mut total_bytes_read = 0;
-
-    loop {
-        match decoder.decode_to_string_without_replacement(&input[total_bytes_read..], output, true)
-        {
-            // If the input is empty, we processed the whole input.
-            (DecoderResult::InputEmpty, _) => break Ok(()),
-            // If the output is full, we must reallocate.
-            (DecoderResult::OutputFull, bytes_read) => {
-                total_bytes_read += bytes_read;
-                // The output is already reserved to the size of the input. We slowly resize. Here,
-                // we're expecting that 10% of bytes will double in size when converting to UTF-8.
-                output.reserve(input.len() / 10);
+    impl<T: std::io::Read> YamlDecoder<T> {
+        /// Create a `YamlDecoder` decoding the given source.
+        pub fn read(source: T) -> YamlDecoder<T> {
+            YamlDecoder {
+                source,
+                trap: YAMLDecodingTrap::Strict,
             }
-            (DecoderResult::Malformed(malformed_len, bytes_after_malformed), bytes_read) => {
-                total_bytes_read += bytes_read;
-                match trap {
-                    // Ignore (skip over) malformed character.
-                    YAMLDecodingTrap::Ignore => {}
-                    // Replace them with the Unicode REPLACEMENT CHARACTER.
-                    YAMLDecodingTrap::Replace => {
-                        output.push('\u{FFFD}');
-                    }
-                    // Otherwise error, getting as much context as possible.
-                    YAMLDecodingTrap::Strict => {
-                        let malformed_len = malformed_len as usize;
-                        let bytes_after_malformed = bytes_after_malformed as usize;
-                        let byte_idx = total_bytes_read - (malformed_len + bytes_after_malformed);
-                        let malformed_sequence = &input[byte_idx..byte_idx + malformed_len];
+        }
 
-                        break Err(LoadError::Decode(Cow::Owned(format!(
-                            "Invalid character sequence at {byte_idx}: {malformed_sequence:?}",
-                        ))));
-                    }
-                    YAMLDecodingTrap::Call(callback) => {
-                        let byte_idx =
-                            total_bytes_read - ((malformed_len + bytes_after_malformed) as usize);
-                        let malformed_sequence =
-                            &input[byte_idx..byte_idx + malformed_len as usize];
-                        if let ControlFlow::Break(error) = callback(
-                            malformed_len,
-                            bytes_after_malformed,
-                            &input[byte_idx..],
-                            output,
-                        ) {
-                            if error.is_empty() {
-                                break Err(LoadError::Decode(Cow::Owned(format!(
+        /// Set the behavior of the decoder when the encoding is invalid.
+        pub fn encoding_trap(&mut self, trap: YAMLDecodingTrap) -> &mut Self {
+            self.trap = trap;
+            self
+        }
+
+        /// Run the decode operation with the source and trap the `YamlDecoder` was built with.
+        ///
+        /// # Errors
+        /// Returns `LoadError` when decoding fails.
+        pub fn decode(&mut self) -> Result<Vec<Yaml>, LoadError> {
+            let mut buffer = Vec::new();
+            self.source.read_to_end(&mut buffer)?;
+
+            // Check if the `encoding` library can detect encoding from the BOM, otherwise use
+            // `detect_utf16_endianness`.
+            let (encoding, _) =
+                Encoding::for_bom(&buffer).unwrap_or_else(|| (detect_utf16_endianness(&buffer), 2));
+            let mut decoder = encoding.new_decoder();
+            let mut output = String::new();
+
+            // Decode the input buffer.
+            decode_loop(&buffer, &mut output, &mut decoder, self.trap)?;
+
+            YamlLoader::load_from_str(&output).map_err(LoadError::Scan)
+        }
+    }
+
+    /// Perform a loop of [`Decoder::decode_to_string`], reallocating `output` if needed.
+    fn decode_loop(
+        input: &[u8],
+        output: &mut String,
+        decoder: &mut Decoder,
+        trap: YAMLDecodingTrap,
+    ) -> Result<(), LoadError> {
+        output.reserve(input.len());
+        let mut total_bytes_read = 0;
+
+        loop {
+            match decoder.decode_to_string_without_replacement(
+                &input[total_bytes_read..],
+                output,
+                true,
+            ) {
+                // If the input is empty, we processed the whole input.
+                (DecoderResult::InputEmpty, _) => break Ok(()),
+                // If the output is full, we must reallocate.
+                (DecoderResult::OutputFull, bytes_read) => {
+                    total_bytes_read += bytes_read;
+                    // The output is already reserved to the size of the input. We slowly resize. Here,
+                    // we're expecting that 10% of bytes will double in size when converting to UTF-8.
+                    output.reserve(input.len() / 10);
+                }
+                (DecoderResult::Malformed(malformed_len, bytes_after_malformed), bytes_read) => {
+                    total_bytes_read += bytes_read;
+                    match trap {
+                        // Ignore (skip over) malformed character.
+                        YAMLDecodingTrap::Ignore => {}
+                        // Replace them with the Unicode REPLACEMENT CHARACTER.
+                        YAMLDecodingTrap::Replace => {
+                            output.push('\u{FFFD}');
+                        }
+                        // Otherwise error, getting as much context as possible.
+                        YAMLDecodingTrap::Strict => {
+                            let malformed_len = malformed_len as usize;
+                            let bytes_after_malformed = bytes_after_malformed as usize;
+                            let byte_idx =
+                                total_bytes_read - (malformed_len + bytes_after_malformed);
+                            let malformed_sequence = &input[byte_idx..byte_idx + malformed_len];
+
+                            break Err(LoadError::Decode(Cow::Owned(format!(
+                                "Invalid character sequence at {byte_idx}: {malformed_sequence:?}",
+                            ))));
+                        }
+                        YAMLDecodingTrap::Call(callback) => {
+                            let byte_idx = total_bytes_read
+                                - ((malformed_len + bytes_after_malformed) as usize);
+                            let malformed_sequence =
+                                &input[byte_idx..byte_idx + malformed_len as usize];
+                            if let ControlFlow::Break(error) = callback(
+                                malformed_len,
+                                bytes_after_malformed,
+                                &input[byte_idx..],
+                                output,
+                            ) {
+                                if error.is_empty() {
+                                    break Err(LoadError::Decode(Cow::Owned(format!(
                                     "Invalid character sequence at {byte_idx}: {malformed_sequence:?}",
                                 ))));
+                                }
+                                break Err(LoadError::Decode(error));
                             }
-                            break Err(LoadError::Decode(error));
                         }
                     }
                 }
             }
         }
     }
-}
 
-/// The encoding crate knows how to tell apart UTF-8 from UTF-16LE and utf-16BE, when the
-/// bytestream starts with BOM codepoint.
-/// However, it doesn't even attempt to guess the UTF-16 endianness of the input bytestream since
-/// in the general case the bytestream could start with a codepoint that uses both bytes.
-///
-/// The YAML-1.2 spec mandates that the first character of a YAML document is an ASCII character.
-/// This allows the encoding to be deduced by the pattern of null (#x00) characters.
-//
-/// See spec at <https://yaml.org/spec/1.2/spec.html#id2771184>
-#[cfg(feature = "encoding")]
-fn detect_utf16_endianness(b: &[u8]) -> &'static Encoding {
-    if b.len() > 1 && (b[0] != b[1]) {
-        if b[0] == 0 {
-            return encoding_rs::UTF_16BE;
-        } else if b[1] == 0 {
-            return encoding_rs::UTF_16LE;
+    /// The encoding crate knows how to tell apart UTF-8 from UTF-16LE and utf-16BE, when the
+    /// bytestream starts with BOM codepoint.
+    /// However, it doesn't even attempt to guess the UTF-16 endianness of the input bytestream since
+    /// in the general case the bytestream could start with a codepoint that uses both bytes.
+    ///
+    /// The YAML-1.2 spec mandates that the first character of a YAML document is an ASCII character.
+    /// This allows the encoding to be deduced by the pattern of null (#x00) characters.
+    //
+    /// See spec at <https://yaml.org/spec/1.2/spec.html#id2771184>
+    fn detect_utf16_endianness(b: &[u8]) -> &'static Encoding {
+        if b.len() > 1 && (b[0] != b[1]) {
+            if b[0] == 0 {
+                return encoding_rs::UTF_16BE;
+            } else if b[1] == 0 {
+                return encoding_rs::UTF_16LE;
+            }
         }
+        encoding_rs::UTF_8
     }
-    encoding_rs::UTF_8
 }
 
 macro_rules! define_as (
@@ -649,6 +670,11 @@ impl Yaml {
     /// assert!(matches!(Yaml::from_str("~"), Yaml::Null));
     /// assert!(matches!(Yaml::from_str("null"), Yaml::Null));
     /// assert!(matches!(Yaml::from_str("true"), Yaml::Boolean(true)));
+    /// assert!(matches!(Yaml::from_str("True"), Yaml::Boolean(true)));
+    /// assert!(matches!(Yaml::from_str("TRUE"), Yaml::Boolean(true)));
+    /// assert!(matches!(Yaml::from_str("false"), Yaml::Boolean(false)));
+    /// assert!(matches!(Yaml::from_str("False"), Yaml::Boolean(false)));
+    /// assert!(matches!(Yaml::from_str("FALSE"), Yaml::Boolean(false)));
     /// assert!(matches!(Yaml::from_str("3.14"), Yaml::Real(_)));
     /// assert!(matches!(Yaml::from_str("foo"), Yaml::String(_)));
     /// ```
@@ -669,8 +695,8 @@ impl Yaml {
         }
         match v {
             "" | "~" | "null" => Yaml::Null,
-            "true" => Yaml::Boolean(true),
-            "false" => Yaml::Boolean(false),
+            "true" | "True" | "TRUE" => Yaml::Boolean(true),
+            "false" | "False" | "FALSE" => Yaml::Boolean(false),
             _ => {
                 if let Ok(integer) = v.parse::<i64>() {
                     Yaml::Integer(integer)
@@ -837,7 +863,7 @@ impl Iterator for YamlIter {
     }
 }
 
-#[cfg(test)]
+#[cfg(all(test, feature = "encoding"))]
 mod test {
     use super::{YAMLDecodingTrap, Yaml, YamlDecoder};
 
